#!/usr/bin/env python

# pip install --upgrade https://robot.inf.um.es/material/umucv.tar.gz
# python pose.py --dev=file:../images/rot4.mjpg

import cv2          as cv
import numpy        as np

from umucv.stream   import mkStream, withKey
from umucv.htrans   import htrans, pose, kgen
from umucv.util     import lineType, cube, showCalib
from umucv.contours import extractContours, redu

import argparse

def readsize(s):
    try:
        return map(int, s.split('x'))
    except:
        raise argparse.ArgumentTypeError("size must be wxh")

parser = argparse.ArgumentParser()
parser.add_argument('--dev', type=str, default='0', help='image source ')
parser.add_argument('--size', help='image size', type=readsize, default=(640,480))
args = parser.parse_args()

imvirt = cv.resize(cv.imread('../images/ccorr/models/thing.png'),(200,200))

stream = mkStream(args.size, args.dev)

HEIGHT, WIDTH = next(stream).shape[:2]
size = WIDTH,HEIGHT
print(size)

K = kgen(size,1.7) # fov aprox 60 degree

print(K)

marker = np.array(
       [[0,   0,   0],
        [0,   1,   0],
        [0.5, 1,   0],
        [0.5, 0.5, 0],
        [1,   0.5, 0],
        [1,   0,   0]])


def polygons(cs,n,prec=2):
    rs = [ redu(c,prec) for c in cs ]
    return [ r for r in rs if len(r) == n ]

def rots(c):
    return [np.roll(c,k,0) for k in range(len(c))]

def bestPose(K,view,model):
    poses = [ pose(K, v.astype(float), model) for v in rots(view) ]
    return sorted(poses,key=lambda p: p[0])[0]



for key,frame in withKey(stream):
    g = cv.cvtColor(frame,cv.COLOR_BGR2GRAY)
    cs = extractContours(g, minarea=5, reduprec=2)

    good = polygons(cs,6,3)
    #print(len(good))

    # como list comprehension:
    #poses = [ Me for err, Me in [bestPose(K,g,marker)] for g in good if err < 2 ] 
    # con un bucle
    poses = []
    for g in good:
        err,Me = bestPose(K,g,marker)
        #print(err,Me)
        if err < 2:
            poses += [Me]
            #print(Me)

    #cv.drawContours(frame,[c.astype(int) for c in cs], -1, (255,255,0), 1, lineType)

    #cv.drawContours(frame,[c.astype(int) for c in good], -1, (0,0,255), 3, lineType)

    #cv.drawContours(frame,[htrans(M,marker).astype(int) for M in poses], -1, (0,255,255), 1, lineType)

    #cv.drawContours(frame,[htrans(M,cube).astype(int) for M in poses], -1, (0,255,0), 1, lineType)

    for p in []: #poses:
        src = np.array([[0.,0],[0,200],[200,200],[200,0]]).astype(np.float32)
        dst = htrans(p,np.array([[0,0,0.5],[1,0,1/2],[1,1,1/2],[0,1,1/2]])).astype(np.float32)
        H = cv.getPerspectiveTransform(src,dst) #(es la homografía plana)
        cv.warpPerspective(imvirt,H,size,frame,0,cv.BORDER_TRANSPARENT)

    showCalib(K,frame)
    cv.imshow('source',frame)
    
